@@ -5,15 +5,15 @@
 from pathlib import Path
 
 import numpy as np
-from scipy import stats
 from nilearn import plotting
+from scipy import stats
 from tqdm import trange
 
 from osl_dynamics import array_ops
-from osl_dynamics.analysis import gmm, static
+from osl_dynamics.analysis import gmm
 from osl_dynamics.analysis.spectral import get_frequency_args_range
-from osl_dynamics.utils.parcellation import Parcellation
 from osl_dynamics.utils.misc import override_dict_defaults
+from osl_dynamics.utils.parcellation import Parcellation
 
 
 def sliding_window_connectivity(
@@ -66,7 +66,7 @@ def sliding_window_connectivity(
 
     # Calculate sliding window connectivity for each subject
     sliding_window_conn = []
-    for i in trange(len(data), desc="Calculating connectivity", ncols=98):
+    for i in trange(len(data), desc="Calculating connectivity"):
         n_samples = data[i].shape[0]
         n_channels = data[i].shape[1]
 
@@ -241,7 +241,6 @@ def mean_coherence_from_spectra(
     # Calculate connectivity for each subject
     c = []
     for i in range(n_subjects):
-
         # Concatenate over modes
         coh = coherence[i].reshape(-1, n_freq)
 
@@ -491,7 +490,6 @@ def fit_gmm(
     percentiles = np.empty([n_components, n_modes])
     for i in range(n_components):
         for j in range(n_modes):
-
             # Off diagonal connectivity values to fit a GMM to
             if subtract_mean:
                 c = conn_map[i, j, m, n] - mean_conn_map[i, m, n]
@@ -531,7 +529,7 @@ def fit_gmm(
                 show_plot=show,
                 plot_filename=plot_filename,
                 plot_kwargs=plot_kwargs,
-                print_message=False,
+                log_message=False,
             )
 
             # Calculate the percentile from the threshold
@@ -672,10 +670,6 @@ def save(
 ):
     """Save connectivity maps.
 
-    If glassbrain=True, this function is a wrapper for
-    nilearn.plotting.view_connectome, otherwise this function is a wrapper for
-    nilearn.plotting.plot_connectome.
-
     Parameters
     ----------
     connectivity_map : np.ndarray
@@ -696,8 +690,22 @@ def save(
         Sholud we create a 3D glass brain plot (as an interactive HTML file)
         or a 2D image plot (as a png, pdf, svg, etc. file).
     plot_kwargs : dict
-        Keyword arguments to pass to the nilearn plotting function.
+        Keyword arguments to pass to the nilearn plotting function:
+
+        - `nilearn.plotting.plot_connectome <https://nilearn.github.io/stable/modules/generated/nilearn.plotting.plot_connectome.html>`_ if glassbrain=False.
+        - `nilearn.plotting.view_connectome <https://nilearn.github.io/stable/modules/generated/nilearn.plotting.view_connectome.html>`_ if glassbrain=True.
+
+        Example use::
+
+            connectivity.save(
+                ...,
+                plot_kwargs={
+                    "edge_cmap": "red_transparent_full_alpha_range",
+                    "display_mode": "lyrz",
+                },
+            )
     """
+
     # Validation
     if filename is not None:
         if glassbrain and Path(filename).suffix != ".html":
@@ -705,6 +713,7 @@ def save(
                 "If glassbrain=True then filename must have a .html extension."
             )
 
+    connectivity_map = np.copy(connectivity_map)
     error_message = (
         "Dimensionality of connectivity_map must be 3 or 4, "
         + f"got ndim={connectivity_map.ndim}."
@@ -736,8 +745,7 @@ def save(
 
     # Loop through each connectivity map
     n_modes = conn_map.shape[0]
-    for i in trange(n_modes, desc="Saving images", ncols=98):
-
+    for i in trange(n_modes, desc="Saving images"):
         # Overwrite keyword arguments if passed
         kwargs = override_dict_defaults(default_plot_kwargs, plot_kwargs)
 
