@@ -2,12 +2,16 @@
 
 """
 
+import logging
+
 import numpy as np
-from scipy import stats, special
+from scipy import special, stats
 from sklearn.mixture import GaussianMixture
 
 from osl_dynamics.utils import plotting
 
+_logger = logging.getLogger("osl-dynamics")
+
 
 def fit_gaussian_mixture(
     X,
@@ -21,7 +25,7 @@ def fit_gaussian_mixture(
     show_plot=False,
     plot_filename=None,
     plot_kwargs={},
-    print_message=True,
+    log_message=True,
 ):
     """Fits a two component Bayesian Gaussian mixture model.
 
@@ -56,16 +60,16 @@ def fit_gaussian_mixture(
     plot_kwargs : dict
         Keyword arguments to pass to plotting function.
         Only used if plot_filename is not None.
-    print_message : bool
-        Should we print a message?
+    log_message : bool
+        Should we log a message?
 
     Returns
     -------
     threshold : float
         Threshold for the on class.
     """
-    if print_message:
-        print("Fitting GMM")
+    if log_message:
+        _logger.info("Fitting GMM")
 
     # Copy the data so we don't modify it
     X_ = np.copy(X)
