@@ -2,17 +2,20 @@
 
 """
 
+import logging
 import warnings
 
 import numpy as np
+from pqdm.processes import pqdm
 from scipy.signal.windows import dpss, hann
 from sklearn.decomposition import non_negative_factorization
 from tqdm import trange
-from pqdm.processes import pqdm
 
 from osl_dynamics import array_ops
 from osl_dynamics.analysis import regression
 
+_logger = logging.getLogger("osl-dynamics")
+
 
 def nextpow2(x):
     """Next power of 2.
@@ -111,7 +114,6 @@ def window_mean(data, window_length, step_size=1, n_sub_windows=1):
     # Array to hold mean of data multiplied by the windowing function
     a = np.empty([n_windows, n_modes], dtype=np.float32)
     for i in range(n_windows):
-
         # Alpha in the window
         j = time_indices[i]
         a_window = data[j : j + window_length]
@@ -138,15 +140,15 @@ def window_mean(data, window_length, step_size=1, n_sub_windows=1):
     return a
 
 
-def coherence_spectra(power_spectra, print_message=True):
+def coherence_spectra(power_spectra, log_message=False):
     """Calculates coherences from (cross) power spectral densities.
 
     Parameters
     ----------
     power_spectra : np.ndarray
         Power spectra. Shape is (n_modes, n_channels, n_channels, n_freq).
-    print_message : bool
-        Should we print a message to screen?
+    log_message : bool
+        Should we log a message to screen?
 
     Returns
     -------
@@ -156,8 +158,8 @@ def coherence_spectra(power_spectra, print_message=True):
     """
     n_modes, n_channels, n_channels, n_freq = power_spectra.shape
 
-    if print_message:
-        print("Calculating coherences")
+    if log_message:
+        _logger.info("Calculating coherences")
 
     coherences = np.empty([n_modes, n_channels, n_channels, n_freq])
     for i in range(n_modes):
@@ -201,7 +203,7 @@ def decompose_spectra(
     components : np.ndarray
         Spectral components. Shape is (n_components, n_freq).
     """
-    print("Performing spectral decomposition")
+    _logger.info("Performing spectral decomposition")
 
     # Validation
     error_message = (
@@ -420,7 +422,7 @@ def mode_covariance_spectra(
         Coherences calculated for each mode. Shape is (n_modes, n_channels,
         n_channels, n_freq).
     """
-    print("Calculating power spectra")
+    _logger.info("Calculating power spectra")
 
     # Validation
     if frequency_range is None:
@@ -494,7 +496,6 @@ def multitaper(
 
     # If tapers are not passed we generate them here
     if tapers is None:
-
         # Check the time half width bandwidth and number of tapers has been passed
         if time_half_bandwidth is None or n_tapers is None:
             raise ValueError("time_half_bandwidth and n_tapers must be passed.")
@@ -591,9 +592,7 @@ def single_multitaper_spectra(
         if parallel:
             iterator = range(n_segments)
         else:
-            if i == 0:
-                print("Calculating spectra:")
-            iterator = trange(n_segments, desc=f"Mode {i}", ncols=98)
+            iterator = trange(n_segments, desc=f"Mode {i}")
 
         for j in iterator:
             # Time series for state i and segment j
@@ -632,7 +631,7 @@ def single_multitaper_spectra(
     p *= n_samples / (sum_alpha * n_tapers * n_segments)
 
     # Coherences for each state
-    c = coherence_spectra(p, print_message=False)
+    c = coherence_spectra(p, log_message=False)
 
     # Only need to keep the diagonal of the power spectra matrix
     p = p[:, range(n_channels), range(n_channels)].real
@@ -770,6 +769,7 @@ def multitaper_spectra(
     if n_subjects == 1:
         # We only have one subject so we don't need to parallelise the
         # calculation
+        _logger.info("Calculating spectra:")
         results = single_multitaper_spectra(
             data[0],
             alpha[0],
@@ -786,6 +786,7 @@ def multitaper_spectra(
         # We have multiple subjects but we're running in serial
         results = []
         for n in range(n_subjects):
+            _logger.info(f"Subject {n}:")
             results.append(
                 single_multitaper_spectra(
                     data[n],
@@ -818,13 +819,12 @@ def multitaper_spectra(
             )
 
         # Calculate spectra in parallel
-        print("Calculating spectra:")
+        _logger.info("Calculating spectra:")
         results = pqdm(
             args,
             single_multitaper_spectra,
             n_jobs=n_jobs,
             argument_type="args",
-            ncols=98,
         )
 
     # Unpack the results
@@ -890,7 +890,7 @@ def single_regression_spectra(
     intercept : np.ndarray
         Regression intercept.
     """
-    print_message = not parallel
+    log_message = not parallel
     t, f, p = spectrogram(
         data,
         window_length,
@@ -899,7 +899,7 @@ def single_regression_spectra(
         calc_cpsd=calc_cpsd,
         step_size=step_size,
         n_sub_windows=n_sub_windows,
-        print_progress_bar=print_message,
+        print_progress_bar=log_message,
     )
     a = window_mean(
         alpha,
@@ -912,7 +912,7 @@ def single_regression_spectra(
         p,
         fit_intercept=True,
         normalize=True,
-        print_message=print_message,
+        log_message=log_message,
     )
     return t, f, coefs, intercept
 
@@ -1042,6 +1042,7 @@ def regression_spectra(
         # We have multiple subjects but we're running in serial
         results = []
         for n in range(n_subjects):
+            _logger.info(f"Subject {n}:")
             results.append(
                 single_regression_spectra(
                     data[n],
@@ -1075,13 +1076,12 @@ def regression_spectra(
             )
 
         # Calculate a time-varying PSD and regress to get the mode PSDs
-        print("Calculating power spectra")
+        _logger.info("Calculating spectra")
         results = pqdm(
             args,
             single_regression_spectra,
             n_jobs=n_jobs,
             argument_type="args",
-            ncols=98,
         )
 
     # Unpack results
@@ -1134,7 +1134,7 @@ def regression_spectra(
 
         # Coherences
         p = np.sum(P[i], axis=0)  # sum coefs and intercept
-        coh[i] = coherence_spectra(p, print_message=False)
+        coh[i] = coherence_spectra(p, log_message=False)
 
     if not return_coef_int:
         # Sum coefficients and intercept
@@ -1233,7 +1233,7 @@ def spectrogram(
             dtype=np.complex64,
         )
         if print_progress_bar:
-            iterator = trange(n_psds, desc="Calculating spectrogram", ncols=98)
+            iterator = trange(n_psds, desc="Calculating spectrogram")
         else:
             iterator = range(n_psds)
         for i in iterator:
@@ -1268,7 +1268,7 @@ def spectrogram(
         P = np.empty([n_psds, n_channels, n_freq], dtype=np.float32)
         XX_sub_window = np.empty([n_sub_windows, n_channels, n_freq], dtype=np.float32)
         if print_progress_bar:
-            iterator = trange(n_psds, desc="Calculating spectrogram", ncols=98)
+            iterator = trange(n_psds, desc="Calculating spectrogram")
         else:
             iterator = range(n_psds)
         for i in iterator:
