@@ -2,6 +2,7 @@
 
 """
 
+import logging
 import pathlib
 import pickle
 import warnings
@@ -10,13 +11,15 @@ from os import path
 from shutil import rmtree
 
 import numpy as np
+from pqdm.threads import pqdm
 from scipy import signal
 from tqdm import tqdm
-from pqdm.threads import pqdm
 
 from osl_dynamics.data import processing, rw, tf
 from osl_dynamics.utils import misc
 
+_logger = logging.getLogger("osl-dynamics")
+
 
 class Data:
     """Data Class.
@@ -30,8 +33,8 @@ class Data:
     inputs : list of str or str or np.ndarray
         - A path to a directory containing .npy files.
           Each .npy file should be a subject or session.
-        - A list of paths to .npy or .mat files. Each file should be a subject or
-          session.
+        - A list of paths to .npy, .mat or .fif files. Each file should be a subject
+          or session. If a .fif file is passed is must end with 'raw.fif' or 'epo.fif'.
         - A numpy array. The array will be treated as continuous data from the
           same subject.
         - A list of numpy arrays. Each numpy array should be the data for a subject
@@ -43,8 +46,20 @@ class Data:
         If a MATLAB (.mat) file is passed, this is the field that corresponds to the
         time series data. By default we read the field 'X'. If a numpy (.npy) file is
         passed, this is ignored. This argument is optional.
+    picks : str or list of str
+        If a fif file is passed we load the data using the MNE object's get_data()
+        method. We pass this argument to the get_data() method. See the
+        `MNE docs <https://mne.tools/stable/generated/mne.io.Raw.html#mne.io.Raw.get_data>`_
+        for further details. This argument is optional. By default picks=None retrieves
+        all channel types.
+    reject_by_annotation : str
+        If a fif file is passed we load the data using the MNE object's get_data()
+        method. If the fif file contains a mne.Raw object, we pass this argument to
+        the get_data() method. This argument is optional. By default
+        reject_by_annotation=None retrieves all time points. Use
+        reject_by_annotation="omit" to remove segments marked as bad.
     sampling_frequency : float
-        Sampling frequency of the data in Hz.  This argument is optional.
+        Sampling frequency of the data in Hz. This argument is optional.
     store_dir : str
         We don't read all the data into memory. Instead we create store them on
         disk and create memmaps (unless load_memmaps=False is passed).
@@ -70,6 +85,8 @@ class Data:
         self,
         inputs,
         data_field="X",
+        picks=None,
+        reject_by_annotation=None,
         sampling_frequency=None,
         store_dir="tmp",
         n_embeddings=None,
@@ -79,6 +96,8 @@ class Data:
     ):
         self._identifier = id(self)
         self.data_field = data_field
+        self.picks = picks
+        self.reject_by_annotation = reject_by_annotation
         self.sampling_frequency = sampling_frequency
         self.n_embeddings = n_embeddings
         self.time_axis_first = time_axis_first
@@ -231,7 +250,6 @@ class Data:
             n_jobs=self.n_jobs,
             desc="Loading files",
             argument_type="args",
-            ncols=98,
             total=len(self.inputs),
         )
 
@@ -255,7 +273,12 @@ class Data:
         if not self.load_memmaps:  # do not load into the memory maps
             mmap_location = None
         raw_data_mmap = rw.load_data(
-            raw_data, self.data_field, mmap_location, mmap_mode="r"
+            raw_data,
+            self.data_field,
+            self.picks,
+            self.reject_by_annotation,
+            mmap_location,
+            mmap_mode="r",
         )
         if not self.time_axis_first:
             raw_data_mmap = raw_data_mmap.T
@@ -273,8 +296,9 @@ class Data:
         output_dir.mkdir(parents=True, exist_ok=True)
 
         # Save time series data
-        for i in tqdm(range(self.n_subjects), desc="Saving data", ncols=98):
-            np.save(f"{output_dir}/subject{i}.npy", self.subjects[i])
+        for i, subject_data in enumerate(tqdm(self.subjects, desc="Saving data")):
+            padded_number = misc.leading_zeros(i, self.n_subjects)
+            np.save(f"{output_dir}/subject{padded_number}.npy", subject_data)
 
         # Save preparation info if .prepared has been called
         if self.prepared:
@@ -316,15 +340,23 @@ class Data:
                 + "Data(..., sampling_frequency=...) when creating the Data object."
             )
 
+        # Save settings
+        self.amplitude_envelope = False
         self.low_freq = low_freq
         self.high_freq = high_freq
+        self.n_window = 1
+        self.n_embeddings = 1
+        self.n_te_channels = self.n_raw_data_channels
+        self.n_pca_components = None
+        self.pca_components = None
+        self.whiten = None
 
         # Create filenames for memmaps (i.e. self.prepared_data_filenames)
         self.prepare_memmap_filenames()
 
         # Prepare the data
         for raw_data_memmap, prepared_data_file in zip(
-            tqdm(self.raw_data_memmaps, desc="Filtering data", ncols=98),
+            tqdm(self.raw_data_memmaps, desc="Filtering data"),
             self.prepared_data_filenames,
         ):
             # Filtering
@@ -456,7 +488,6 @@ class Data:
             desc="Preparing data",
             n_jobs=self.n_jobs,
             argument_type="args",
-            ncols=98,
             total=len(self.raw_data_memmaps),
         )
         self.prepared_data_memmaps.extend(prepared_data_memmaps)
@@ -561,12 +592,11 @@ class Data:
         # Principle component analysis (PCA)
         # NOTE: the approach used here only works for zero mean data
         if n_pca_components is not None:
-
             # Calculate the PCA components by performing SVD on the covariance
             # of the data
             covariance = np.zeros([self.n_te_channels, self.n_te_channels])
             for raw_data_memmap in tqdm(
-                self.raw_data_memmaps, desc="Calculating PCA components", ncols=98
+                self.raw_data_memmaps, desc="Calculating PCA components"
             ):
                 # Standardise and time embed the data
                 std_data = processing.standardize(raw_data_memmap)
@@ -579,7 +609,7 @@ class Data:
             u, s, vh = np.linalg.svd(covariance)
             u = u[:, :n_pca_components].astype(np.float32)
             explained_variance = np.sum(s[:n_pca_components]) / np.sum(s)
-            print(f"Explained variance: {100 * explained_variance:.1f}%")
+            _logger.info(f"Explained variance: {100 * explained_variance:.1f}%")
             s = s[:n_pca_components].astype(np.float32)
             if whiten:
                 u = u @ np.diag(1.0 / np.sqrt(s))
@@ -596,7 +626,6 @@ class Data:
             desc="Preparing data",
             n_jobs=self.n_jobs,
             argument_type="args",
-            ncols=98,
             total=len(self.raw_data_memmaps),
         )
         self.prepared_data_memmaps.extend(prepared_data_memmaps)
@@ -706,7 +735,6 @@ class Data:
 
         trimmed_time_series = []
         for memmap in memmaps:
-
             # Remove data points lost to time embedding
             if n_embeddings != 1:
                 memmap = memmap[n_embeddings // 2 : -(n_embeddings // 2)]
@@ -884,7 +912,7 @@ class Data:
                 # Split the full dataset into a training and validation dataset
                 training_dataset = full_dataset.take(training_dataset_size)
                 validation_dataset = full_dataset.skip(training_dataset_size)
-                print(
+                _logger.info(
                     f"{len(training_dataset)} batches in training dataset, "
                     + f"{len(validation_dataset)} batches in the validation dataset."
                 )
@@ -917,7 +945,6 @@ class Data:
                 training_datasets = []
                 validation_datasets = []
                 for i in range(len(full_datasets)):
-
                     # Calculate the number of batches in the training dataset
                     dataset_size = len(full_datasets[i])
                     training_dataset_size = round(
@@ -931,7 +958,7 @@ class Data:
                     validation_datasets.append(
                         full_datasets[i].skip(training_dataset_size)
                     )
-                    print(
+                    _logger.info(
                         f"Subject {i}: "
                         + f"{len(training_datasets[i])} batches in training dataset, "
                         + f"{len(validation_datasets[i])} batches in the validation dataset."
