@@ -2,6 +2,7 @@
 
 """
 
+import logging
 import time
 from dataclasses import dataclass
 from typing import Literal
@@ -10,20 +11,22 @@ import numpy as np
 from tensorflow.keras import layers, models, optimizers, utils
 from tqdm import trange
 
+from osl_dynamics.inference.layers import (
+    AdversarialLogLikelihoodLossLayer,
+    ConcatVectorsMatricesLayer,
+    CorrelationMatricesLayer,
+    DiagonalMatricesLayer,
+    InferenceRNNLayer,
+    MatMulLayer,
+    MixMatricesLayer,
+    MixVectorsLayer,
+    ModelRNNLayer,
+    VectorsLayer,
+)
 from osl_dynamics.models import mdynemo_obs
 from osl_dynamics.models.mod_base import BaseModelConfig, ModelBase
-from osl_dynamics.inference.layers import (
-    InferenceRNNLayer,
-    VectorsLayer,
-    DiagonalMatricesLayer,
-    CorrelationMatricesLayer,
-    MixVectorsLayer,
-    MixMatricesLayer,
-    MatMulLayer,
-    ModelRNNLayer,
-    ConcatVectorsMatricesLayer,
-    AdversarialLogLikelihoodLossLayer,
-)
+
+_logger = logging.getLogger("osl-dynamics")
 
 
 @dataclass
@@ -207,7 +210,7 @@ class Model(ModelBase):
         """Builds a keras model for the inference, generator and discriminator model
         and the full MAGE model.
         """
-        print("Build models")
+        _logger.info("Build models")
         self.inference_model = _build_inference_model(self.config)
         self.inference_model.summary()
         print()
@@ -375,7 +378,7 @@ class Model(ModelBase):
 
             if generator_loss[0] < best_val_loss:
                 self.save_weights(save_filepath)
-                print(
+                _logger.info(
                     "Best model w/ val loss (generator) {} saved to {}".format(
                         generator_loss[0], save_filepath
                     )
@@ -392,7 +395,7 @@ class Model(ModelBase):
             )
 
         # Load the weights for the best model
-        print("Loading best model:", save_filepath)
+        _logger.info(f"Loading best model: {save_filepath}")
         self.load_weights(save_filepath)
 
         return history
@@ -438,7 +441,7 @@ class Model(ModelBase):
 
         inputs = self.make_dataset(inputs, concatenate=concatenate)
 
-        print("Getting mode time courses:")
+        _logger.info("Getting mode time courses:")
         outputs_alpha = []
         outputs_gamma = []
         for dataset in inputs:
@@ -481,7 +484,6 @@ class Model(ModelBase):
         for i in trange(
             n_samples - self.config.sequence_length,
             desc="Predicting mode time course",
-            ncols=98,
         ):
             # Extract the sequence
             alpha_input = alpha[i : i + self.config.sequence_length]
