@@ -2,6 +2,7 @@
 
 """
 
+import logging
 from dataclasses import dataclass
 from typing import Literal
 
@@ -10,29 +11,31 @@ import tensorflow as tf
 from tensorflow.keras import layers
 from tqdm import trange
 
-from osl_dynamics.models import dynemo_obs, mdynemo_obs
-from osl_dynamics.models.mod_base import BaseModelConfig
-from osl_dynamics.models.inf_mod_base import (
-    VariationalInferenceModelConfig,
-    VariationalInferenceModelBase,
-)
 from osl_dynamics.inference.layers import (
-    InferenceRNNLayer,
-    LogLikelihoodLossLayer,
-    VectorsLayer,
-    DiagonalMatricesLayer,
+    ConcatenateLayer,
     CorrelationMatricesLayer,
-    MixVectorsLayer,
-    MixMatricesLayer,
-    ModelRNNLayer,
-    NormalizationLayer,
+    DiagonalMatricesLayer,
+    InferenceRNNLayer,
     KLDivergenceLayer,
     KLLossLayer,
+    LogLikelihoodLossLayer,
+    MatMulLayer,
+    MixMatricesLayer,
+    MixVectorsLayer,
+    ModelRNNLayer,
+    NormalizationLayer,
     SampleNormalDistributionLayer,
     SoftmaxLayer,
-    ConcatenateLayer,
-    MatMulLayer,
+    VectorsLayer,
 )
+from osl_dynamics.models import dynemo_obs, mdynemo_obs
+from osl_dynamics.models.inf_mod_base import (
+    VariationalInferenceModelBase,
+    VariationalInferenceModelConfig,
+)
+from osl_dynamics.models.mod_base import BaseModelConfig
+
+_logger = logging.getLogger("osl-dynamics")
 
 
 @dataclass
@@ -219,7 +222,7 @@ class Config(BaseModelConfig, VariationalInferenceModelConfig):
         super().validate_dimension_parameters()
         if self.n_fc_modes is None:
             self.n_fc_modes = self.n_modes
-            print("Warning: n_fc_modes is None, set to n_modes.")
+            _logger.warning("n_fc_modes is None, set to n_modes.")
 
 
 class Model(VariationalInferenceModelBase):
@@ -302,10 +305,14 @@ class Model(VariationalInferenceModelBase):
             dynemo_obs.set_means_regularizer(self.model, training_dataset)
 
         if self.config.learn_stds:
-            mdynemo_obs.set_stds_regularizer(self.model, training_dataset)
+            mdynemo_obs.set_stds_regularizer(
+                self.model, training_dataset, self.config.stds_epsilon
+            )
 
         if self.config.learn_fcs:
-            mdynemo_obs.set_fcs_regularizer(self.model, training_dataset)
+            mdynemo_obs.set_fcs_regularizer(
+                self.model, training_dataset, self.config.fcs_epsilon
+            )
 
     def sample_time_courses(self, n_samples: int):
         """Uses the model RNN to sample mode mixing factors, alpha and gamma.
@@ -357,7 +364,7 @@ class Model(VariationalInferenceModelBase):
         # Sample the mode time courses
         alpha = np.empty([n_samples, self.config.n_modes])
         gamma = np.empty([n_samples, self.config.n_fc_modes])
-        for i in trange(n_samples, desc="Sampling mode time courses", ncols=98):
+        for i in trange(n_samples, desc="Sampling mode time courses"):
             # If there are leading zeros we trim theta so that we don't pass the zeros
             trimmed_mean_theta = mean_theta_norm[~np.all(mean_theta_norm == 0, axis=1)][
                 np.newaxis, :, :
@@ -392,7 +399,6 @@ class Model(VariationalInferenceModelBase):
 
 
 def _model_structure(config):
-
     # Layer for input
     inputs = layers.Input(
         shape=(config.sequence_length, config.n_channels), name="data"
