@@ -2,15 +2,16 @@
 
 """
 
+import logging
 import os
 import re
 from abc import abstractmethod
-from io import StringIO
 from dataclasses import dataclass
+from io import StringIO
 
 import numpy as np
-import yaml
 import tensorflow
+import yaml
 from tensorflow.data import Dataset
 from tensorflow.keras import optimizers
 from tensorflow.python.distribute.distribution_strategy_context import get_strategy
@@ -21,9 +22,11 @@ from tqdm.keras import TqdmCallback
 import osl_dynamics
 from osl_dynamics import data
 from osl_dynamics.inference import callbacks, initializers
-from osl_dynamics.utils.misc import get_argument, replace_argument, NumpyLoader
+from osl_dynamics.utils.misc import NumpyLoader, get_argument, replace_argument
 from osl_dynamics.utils.model import HTMLTable, LatexTable
 
+_logger = logging.getLogger("osl-dynamics")
+
 
 @dataclass
 class BaseModelConfig:
@@ -202,15 +205,8 @@ class ModelBase:
 
         # Callback to display a progress bar with tqdm
         if use_tqdm:
-            if tqdm_class is not None:
-                tqdm_callback = TqdmCallback(verbose=0, tqdm_class=tqdm_class)
-            else:
-                # Create a tqdm class with a progress bar width of 98 characters
-                class tqdm_class(tqdm_auto):
-                    def __init__(self, *args, **kwargs):
-                        super().__init__(*args, **kwargs, ncols=98)
-
-                tqdm_callback = TqdmCallback(verbose=0, tqdm_class=tqdm_class)
+            tqdm_class = tqdm_class or tqdm_auto
+            tqdm_callback = TqdmCallback(verbose=0, tqdm_class=tqdm_class)
             additional_callbacks.append(tqdm_callback)
 
         # Callback to save the best model after a certain number of epochs
@@ -460,10 +456,19 @@ class ModelBase:
         model : Model
             Model object.
         """
-        print("Loading model:", dirname)
+        _logger.info(f"Loading model: {dirname}")
 
-        # Load config
+        # Load config dict and version from yml file
         config_dict, version = cls.load_config(dirname)
+
+        if version in ["<1.1.6", "1.1.6", "1.1.7"]:
+            raise ValueError(
+                f"model was trained using osl-dynamics version {version}. "
+                + "This is incompatible with the current version. "
+                + "Please revert to v1.1.7 to load this model."
+            )
+
+        # Create config object
         config = cls.config_type(**config_dict)
 
         # Create model
