@@ -2,6 +2,7 @@
 
 """
 
+import logging
 import time
 from dataclasses import dataclass
 from typing import Literal
@@ -10,18 +11,20 @@ import numpy as np
 from tensorflow.keras import layers, models, optimizers, utils
 from tqdm import trange
 
+from osl_dynamics.inference.layers import (
+    AdversarialLogLikelihoodLossLayer,
+    ConcatVectorsMatricesLayer,
+    CovarianceMatricesLayer,
+    InferenceRNNLayer,
+    MixMatricesLayer,
+    MixVectorsLayer,
+    ModelRNNLayer,
+    VectorsLayer,
+)
 from osl_dynamics.models import dynemo_obs
 from osl_dynamics.models.mod_base import BaseModelConfig, ModelBase
-from osl_dynamics.inference.layers import (
-    InferenceRNNLayer,
-    VectorsLayer,
-    CovarianceMatricesLayer,
-    MixVectorsLayer,
-    MixMatricesLayer,
-    ModelRNNLayer,
-    ConcatVectorsMatricesLayer,
-    AdversarialLogLikelihoodLossLayer,
-)
+
+_logger = logging.getLogger("osl-dynamics")
 
 
 @dataclass
@@ -177,7 +180,7 @@ class Model(ModelBase):
         """Builds a keras model for the inference, generator and discriminator model
         and the full SAGE model.
         """
-        print("Build models")
+        _logger.info("Build models")
         self.inference_model = _build_inference_model(self.config)
         self.inference_model.summary()
         print()
@@ -297,7 +300,7 @@ class Model(ModelBase):
 
             if generator_loss[0] < best_val_loss:
                 self.save_weights(save_filepath)
-                print(
+                _logger.info(
                     "Best model w/ val loss (generator) {} saved to {}".format(
                         generator_loss[0], save_filepath
                     )
@@ -308,7 +311,7 @@ class Model(ModelBase):
             history.append({"D": discriminator_loss[1], "G": generator_loss[0]})
 
         # Load the weights for the best model
-        print("Loading best model:", save_filepath)
+        _logger.info(f"Loading best model: {save_filepath}")
         self.load_weights(save_filepath)
 
         return history
@@ -340,7 +343,7 @@ class Model(ModelBase):
         """
         inputs = self.make_dataset(inputs, concatenate=concatenate)
 
-        print("Getting alpha:")
+        _logger.info("Getting alpha:")
         outputs = []
         for dataset in inputs:
             alpha = self.inference_model.predict(dataset)[1]
@@ -421,7 +424,6 @@ class Model(ModelBase):
         for i in trange(
             n_samples - self.config.sequence_length,
             desc="Predicting mode time course",
-            ncols=98,
         ):
             # Extract the sequence
             alpha_input = alpha[i : i + self.config.sequence_length]
