@@ -1,4 +1,5 @@
-"""Subject Embedding Dynamic Network Modes (SE-DyNeMo).
+"""Subject-Embedding DyNeMo (SE-DyNeMo).
+
 """
 
 from dataclasses import dataclass
@@ -6,7 +7,7 @@ from typing import Literal
 
 import numpy as np
 import tensorflow as tf
-from tensorflow.keras import layers
+from tensorflow.keras import layers, initializers
 
 import osl_dynamics.data.tf as dtf
 from osl_dynamics.models import dynemo_obs, sedynemo_obs
@@ -25,16 +26,18 @@ from osl_dynamics.inference.layers import (
     KLDivergenceLayer,
     KLLossLayer,
     SampleNormalDistributionLayer,
+    SampleGammaDistributionLayer,
     SoftmaxLayer,
     ConcatenateLayer,
-    SubjectDevEmbeddingLayer,
+    ConcatEmbeddingsLayer,
     SubjectMapLayer,
-    MixSubjectEmbeddingParametersLayer,
+    MixSubjectSpecificParametersLayer,
     TFRangeLayer,
     ZeroLayer,
     InverseCholeskyLayer,
-    SubjectMapKLDivergenceLayer,
+    StaticKLDivergenceLayer,
     MultiLayerPerceptronLayer,
+    LearnableTensorLayer,
 )
 
 
@@ -137,33 +140,21 @@ class Config(BaseModelConfig, VariationalInferenceModelConfig):
 
     n_subjects : int
         Number of subjects.
-    subject_embedding_dim : int
+    subject_embeddings_dim : int
         Number of dimensions for the subject embedding.
-    mode_embedding_dim : int
+    mode_embeddings_dim : int
         Number of dimensions for the mode embedding in the spatial maps encoder.
 
-    dev_inf_n_layers : int
-        Number of layers for the inference MLP for deviations.
-    dev_inf_n_units : int
-        Number of units for the inference MLP for deviations.
-    dev_inf_normalization : str
-        Type of normalization for the inference MLP for deviations.
-        Either None, 'batch' or 'layer'.
-    dev_inf_activation : str
-        Type of activation to use for the inference MLP for deviations.
-        E.g. 'relu', 'sigmoid', 'tanh', etc.
-    dev_inf_dropout : float
-        Dropout rate for the inference MLP for deviations.
-    dev_mod_n_layers : int
-        Number of layers for the model MLP for deviations.
-    dev_mod_n_units : int
-        Number of units for the model MLP for deviations.
-    dev_mod_normalization : str
-        Type of normalization for the model MLP for deviations.
-    dev_mod_activation : str
-        Type of activation to use for the model MLP for deviations.
-    dev_mod_dropout : float
-        Dropout rate for the model MLP for deviations.
+    dev_n_layers : int
+        Number of layers for the MLP for deviations.
+    dev_n_units : int
+        Number of units for the MLP for deviations.
+    dev_normalization : str
+        Type of normalization for the MLP for deviations.
+    dev_activation : str
+        Type of activation to use for the MLP for deviations.
+    dev_dropout : float
+        Dropout rate for the MLP for deviations.
     """
 
     model_name: str = "SE-DyNeMo"
@@ -197,19 +188,14 @@ class Config(BaseModelConfig, VariationalInferenceModelConfig):
 
     # Parameters specific to subject embedding model
     n_subjects: int = None
-    subject_embedding_dim: int = None
-    mode_embedding_dim: int = None
+    subject_embeddings_dim: int = None
+    mode_embeddings_dim: int = None
 
-    dev_inf_n_layers: int = 0
-    dev_inf_n_units: int = None
-    dev_inf_normalization: str = None
-    dev_inf_activation: str = None
-    dev_inf_dropout: float = 0.0
-    dev_mod_n_layers: int = 0
-    dev_mod_n_units: int = None
-    dev_mod_normalization: str = None
-    dev_mod_activation: str = None
-    dev_mod_dropout: float = 0.0
+    dev_n_layers: int = 0
+    dev_n_units: int = None
+    dev_normalization: str = None
+    dev_activation: str = None
+    dev_dropout: float = 0.0
 
     def __post_init__(self):
         self.validate_rnn_parameters()
@@ -240,26 +226,23 @@ class Config(BaseModelConfig, VariationalInferenceModelConfig):
     def validate_subject_embedding_parameters(self):
         if (
             self.n_subjects is None
-            or self.subject_embedding_dim is None
-            or self.mode_embedding_dim is None
+            or self.subject_embeddings_dim is None
+            or self.mode_embeddings_dim is None
         ):
             raise ValueError(
                 "n_subjects, subject_embedding_dim and mode_embedding_dim must be passed."
             )
 
-        if self.dev_inf_n_layers != 0 and self.dev_inf_n_units is None:
-            raise ValueError("Please pass dev_inf_n_units.")
-
-        if self.dev_mod_n_layers != 0 and self.dev_mod_n_units is None:
-            raise ValueError("Plase pass dev_mod_n_units.")
+        if self.dev_n_layers != 0 and self.dev_n_units is None:
+            raise ValueError("Please pass dev_n_units.")
 
 
 class Model(VariationalInferenceModelBase):
-    """SE-DyNeMo model class.
+    """Directional Subject embedding DyNeMo.
 
     Parameters
     ----------
-    config : dynemo.models.sedynemo.Config
+    config : osl_dynamics_models.sedynemo.Config
     """
 
     config_type = Config
@@ -282,7 +265,7 @@ class Model(VariationalInferenceModelBase):
         covariances : np.ndarray
             Mode covariances for the group. Shape is (n_modes, n_channels, n_channels).
         """
-        return sedynemo_obs.get_group_means_covariances(self.model)
+        return sedynemo_obs.get_group_means_covs(self.model)
 
     def get_observation_model_parameters(self):
         """Wrapper for get_group_means_covariances."""
@@ -294,51 +277,20 @@ class Model(VariationalInferenceModelBase):
         Returns
         -------
         subject_embeddings : np.ndarray
-            Embedding vectors for subjects. Shape is (n_subjects, subject_embedding_dim).
+            Embedding vectors for subjects.
+            Shape is (n_subjects, subject_embedding_dim).
         """
         return sedynemo_obs.get_subject_embeddings(self.model)
 
-    def get_mode_embeddings(self):
-        """Get the mode spatial map embeddings.
+    def get_subject_means_covariances(self, subject_embeddings=None, n_neighbours=2):
+        """Get the means and covariances for each subject.
 
-        Returns
-        -------
-        means_mode_embeddings : np.ndarray
-            Mode embeddings for means. Shape is (n_modes, mode_embedding_dim).
-        covs_mode_embeddings : np.ndarray
-            Mode embeddings for covs. Shape is (n_modes, mode_embedding_dim).
-        """
-        return sedynemo_obs.get_mode_embeddings(self.model)
-
-    def get_concatenated_embeddings(self):
-        """Get the concatenated embedding vectors of deviations.
-
-        Returns
-        -------
-        means_embedding : np.ndarray
-            Embedding vectors for the mean deviations.
-            Shape is (n_subjects, n_modes, subject_embedding_dim + mode_embedding_dim).
-        covs_embedding : np.ndarray
-            Embedding vectors for the covs deviations.
-            Shape is (n_subjects, n_modes, subject_embedding_dim + mode_embedding_dim).
-        """
-        return sedynemo_obs.get_concatenated_embeddings(self.model)
-
-    def get_subject_dev(self):
-        """Get the subject specific deviations of means and covs from the group.
-
-        Returns
-        -------
-        means_dev : np.ndarray
-            Deviation of means from the group. Shape is (n_subjects, n_modes, n_channels).
-        covs_dev : np.ndarray
-            Deviation of Cholesky factor of covs from the group.
-            Shape is (n_subjects, n_modes, n_channels * (n_channels + 1) // 2).
-        """
-        return sedynemo_obs.get_subject_dev(self.model)
-
-    def get_subject_means_covariances(self):
-        """Get the means and covariances for each subject
+        Parameters
+        ----------
+        subject_embeddings : np.ndarray
+            Input embedding vectors for subjects. Shape is (n_subjects, subject_embeddings_dim).
+        n_neighbours : int
+            Number of nearest neighbours. Ignored if subject_embedding=None.
 
         Returns
         -------
@@ -348,7 +300,13 @@ class Model(VariationalInferenceModelBase):
             Mode covariances for each subject.
             Shape is (n_subjects, n_modes, n_channels, n_channels).
         """
-        return sedynemo_obs.get_subject_means_covariances(self.model)
+        return sedynemo_obs.get_subject_means_covs(
+            self.model,
+            self.config.learn_means,
+            self.config.learn_covariances,
+            subject_embeddings,
+            n_neighbours,
+        )
 
     def set_regularizers(self, training_dataset):
         """Set the means and covariances regularizer based on the training data.
@@ -371,7 +329,10 @@ class Model(VariationalInferenceModelBase):
 
         if self.config.learn_covariances:
             dynemo_obs.set_covariances_regularizer(
-                self.model, training_dataset, layer_name="group_covs"
+                self.model,
+                training_dataset,
+                self.config.covariances_epsilon,
+                layer_name="group_covs",
             )
 
     def set_group_means(self, group_means, update_initializer=True):
@@ -400,8 +361,8 @@ class Model(VariationalInferenceModelBase):
             the model?
         """
         dynemo_obs.set_covariances(
-            model=self.model,
-            covariances=group_covariances,
+            self.model,
+            group_covariances,
             update_initializer=update_initializer,
             layer_name="group_covs",
         )
@@ -435,30 +396,21 @@ class Model(VariationalInferenceModelBase):
             self.model, n_batches, learn_means, learn_covariances
         )
 
-    def random_subject_initialization(
-        self, training_data, n_epochs, n_subjects, n_kl_annealing_epochs=None, **kwargs
-    ):
-        """Random subject initialisation.
-
-        Currently not implemented in osl-dynamics. This method will raise
-        an Attribute error.
-        """
+    def random_subject_initialization(self, **kwargs):
+        """random subject initialisation not compatible with SE-DyNeMo."""
         raise AttributeError(
-            "'Model' object has no attribute 'random_subject_initialization'."
+            " 'Model' object has no attribute 'random_subject_initialization'."
         )
 
 
 def _model_structure(config):
-    # layers for inputs
+
+    # Layers for inputs
     data = layers.Input(shape=(config.sequence_length, config.n_channels), name="data")
     subj_id = layers.Input(shape=(config.sequence_length,), name="subj_id")
 
     # Inference RNN:
-    # - Learns q(theta) ~ N(theta | inf_mu, inf_sigma), where
-    #     - inf_mu    ~ affine(RNN(data_<=t))
-    #     - inf_sigma ~ softplus(RNN(data_<=t))
-
-    # Definition of layers
+    # Layer definitions
     inference_input_dropout_layer = layers.Dropout(
         config.inference_dropout, name="data_drop"
     )
@@ -496,22 +448,16 @@ def _model_structure(config):
     theta_norm = theta_norm_layer(theta)
     alpha = alpha_layer(theta_norm)
 
-    # Observation model:
-    # - We use a multivariate normal with a mean vector and covariance matrix for
-    #   each mode as the observation model.
-    # - Each subject has their own mean vectors and covariance matrices for each mode.
-    #   They are near the group means and covariances.
-    # - We calculate the likelihood of generating the training data with alpha
-    #   and the observation model.
+    # -----------------
+    # Observation model
 
-    # Definition of layers
-
-    # Subject embedding layer
+    # Subject embedding layers
     subjects_layer = TFRangeLayer(config.n_subjects, name="subjects")
-    subject_embedding_layer = layers.Embedding(
-        config.n_subjects, config.subject_embedding_dim, name="subject_embeddings"
+    subject_embeddings_layer = layers.Embedding(
+        config.n_subjects, config.subject_embeddings_dim, name="subject_embeddings"
     )
 
+    # Group level observation model parameters
     group_means_layer = VectorsLayer(
         config.n_modes,
         config.n_channels,
@@ -529,70 +475,175 @@ def _model_structure(config):
         config.covariances_regularizer,
         name="group_covs",
     )
-    means_mode_embedding_layer = layers.Dense(
-        config.mode_embedding_dim,
-        name="means_mode_embedding",
-    )
-    covs_mode_embedding_layer = layers.Dense(
-        config.mode_embedding_dim,
-        name="covs_mode_embedding",
-    )
-    means_concat_embedding_layer = SubjectDevEmbeddingLayer(
-        config.n_modes,
-        config.n_channels,
-        config.n_subjects,
-        name="means_concat_embedding",
-    )
-    covs_concat_embedding_layer = SubjectDevEmbeddingLayer(
-        config.n_modes,
-        config.n_channels,
-        config.n_subjects,
-        name="covs_concat_embedding",
-    )
 
-    # Inference part of the deviation
-    means_dev_inf_input_layer = MultiLayerPerceptronLayer(
-        config.dev_inf_n_layers,
-        config.dev_inf_n_units,
-        config.dev_inf_normalization,
-        config.dev_inf_activation,
-        config.dev_inf_dropout,
-        name="means_dev_inf_input",
-    )
-    means_dev_inf_mu_layer = layers.Dense(config.n_channels, name="means_dev_inf_mu")
-    means_dev_inf_sigma_layer = layers.Dense(
-        config.n_channels, activation="softplus", name="means_dev_inf_sigma"
-    )
+    subjects = subjects_layer(data)
+    subject_embeddings = subject_embeddings_layer(subjects)
+
+    group_mu = group_means_layer(data)
+    group_D = group_covs_layer(data)
+
+    # ---------------
+    # Mean deviations
+
+    # Layer definitions
     if config.learn_means:
-        means_dev_layer = SampleNormalDistributionLayer(
-            config.theta_std_epsilon, name="means_dev"
+        means_mode_embeddings_layer = layers.Dense(
+            config.mode_embeddings_dim,
+            name="means_mode_embeddings",
         )
+        means_concat_embeddings_layer = ConcatEmbeddingsLayer(
+            name="means_concat_embeddings",
+        )
+
+        means_dev_map_input_layer = MultiLayerPerceptronLayer(
+            config.dev_n_layers,
+            config.dev_n_units,
+            config.dev_normalization,
+            config.dev_activation,
+            config.dev_dropout,
+            name="means_dev_map_input",
+        )
+        means_dev_map_layer = layers.Dense(config.n_channels, name="means_dev_map")
+
+        norm_means_dev_map_layer = layers.LayerNormalization(
+            axis=-1, name="norm_means_dev_map"
+        )
+
+        means_dev_mag_inf_alpha_input_layer = LearnableTensorLayer(
+            shape=(config.n_subjects, config.n_modes, 1),
+            learn=config.learn_means,
+            initializer=initializers.TruncatedNormal(mean=0, stddev=0.02),
+            name="means_dev_mag_inf_alpha_input",
+        )
+        means_dev_mag_inf_alpha_layer = layers.Activation(
+            "softplus", name="means_dev_mag_inf_alpha"
+        )
+        means_dev_mag_inf_beta_input_layer = LearnableTensorLayer(
+            shape=(config.n_subjects, config.n_modes, 1),
+            learn=config.learn_means,
+            initializer=initializers.TruncatedNormal(mean=0, stddev=0.02),
+            name="means_dev_mag_inf_beta_input",
+        )
+        means_dev_mag_inf_beta_layer = layers.Activation(
+            "softplus", name="means_dev_mag_inf_beta"
+        )
+        means_dev_mag_layer = SampleGammaDistributionLayer(
+            config.covariances_epsilon, name="means_dev_mag"
+        )
+
+        means_dev_layer = layers.Multiply(name="means_dev")
+
+        # Data flow to get the subject specific deviations of means
+
+        # Get the concatenated embeddings
+        means_mode_embeddings = means_mode_embeddings_layer(group_mu)
+        means_concat_embeddings = means_concat_embeddings_layer(
+            [subject_embeddings, means_mode_embeddings]
+        )
+
+        # Get the mean deviation maps (no global magnitude information)
+        means_dev_map_input = means_dev_map_input_layer(means_concat_embeddings)
+        means_dev_map = means_dev_map_layer(means_dev_map_input)
+        norm_means_dev_map = norm_means_dev_map_layer(means_dev_map)
+
+        # Get the deviation magnitudes (scale deviation maps globally)
+
+        means_dev_mag_inf_alpha_input = means_dev_mag_inf_alpha_input_layer(data)
+        means_dev_mag_inf_alpha = means_dev_mag_inf_alpha_layer(
+            means_dev_mag_inf_alpha_input
+        )
+        means_dev_mag_inf_beta_input = means_dev_mag_inf_beta_input_layer(data)
+        means_dev_mag_inf_beta = means_dev_mag_inf_beta_layer(
+            means_dev_mag_inf_beta_input
+        )
+        means_dev_mag = means_dev_mag_layer(
+            [means_dev_mag_inf_alpha, means_dev_mag_inf_beta]
+        )
+        means_dev = means_dev_layer([means_dev_mag, norm_means_dev_map])
     else:
         means_dev_layer = ZeroLayer(
             shape=(config.n_subjects, config.n_modes, config.n_channels),
             name="means_dev",
         )
+        means_dev = means_dev_layer(data)
 
-    covs_dev_inf_input_layer = MultiLayerPerceptronLayer(
-        config.dev_inf_n_layers,
-        config.dev_inf_n_units,
-        config.dev_inf_normalization,
-        config.dev_inf_activation,
-        config.dev_inf_dropout,
-        name="covs_dev_inf_input",
-    )
-    covs_dev_inf_mu_layer = layers.Dense(
-        config.n_channels * (config.n_channels + 1) // 2, name="covs_dev_inf_mu"
-    )
-    covs_dev_inf_sigma_layer = layers.Dense(
-        config.n_channels * (config.n_channels + 1) // 2,
-        activation="softplus",
-        name="covs_dev_inf_sigma",
-    )
+    # ----------------------
+    # Covariances deviations
+
+    # Layer definitions
     if config.learn_covariances:
-        covs_dev_layer = SampleNormalDistributionLayer(
-            config.theta_std_epsilon, name="covs_dev"
+        covs_mode_embeddings_layer = layers.Dense(
+            config.mode_embeddings_dim,
+            name="covs_mode_embeddings",
         )
+        covs_concat_embeddings_layer = ConcatEmbeddingsLayer(
+            name="covs_concat_embeddings",
+        )
+
+        covs_dev_map_input_layer = MultiLayerPerceptronLayer(
+            config.dev_n_layers,
+            config.dev_n_units,
+            config.dev_normalization,
+            config.dev_activation,
+            config.dev_dropout,
+            name="covs_dev_map_input",
+        )
+        covs_dev_map_layer = layers.Dense(
+            config.n_channels * (config.n_channels + 1) // 2, name="covs_dev_map"
+        )
+        norm_covs_dev_map_layer = layers.LayerNormalization(
+            axis=-1, name="norm_covs_dev_map"
+        )
+
+        covs_dev_mag_inf_alpha_input_layer = LearnableTensorLayer(
+            shape=(config.n_subjects, config.n_modes, 1),
+            learn=config.learn_covariances,
+            initializer=initializers.TruncatedNormal(mean=0, stddev=0.02),
+            name="covs_dev_mag_inf_alpha_input",
+        )
+        covs_dev_mag_inf_alpha_layer = layers.Activation(
+            "softplus", name="covs_dev_mag_inf_alpha"
+        )
+        covs_dev_mag_inf_beta_input_layer = LearnableTensorLayer(
+            shape=(config.n_subjects, config.n_modes, 1),
+            learn=config.learn_covariances,
+            initializer=initializers.TruncatedNormal(mean=0, stddev=0.02),
+            name="covs_dev_mag_inf_beta_input",
+        )
+        covs_dev_mag_inf_beta_layer = layers.Activation(
+            "softplus", name="covs_dev_mag_inf_beta"
+        )
+        covs_dev_mag_layer = SampleGammaDistributionLayer(
+            config.covariances_epsilon, name="covs_dev_mag"
+        )
+        covs_dev_layer = layers.Multiply(name="covs_dev")
+
+        # Data flow to get subject specific deviations of covariances
+
+        # Get the concatenated embeddings
+        covs_mode_embeddings = covs_mode_embeddings_layer(
+            InverseCholeskyLayer(config.covariances_epsilon)(group_D)
+        )
+        covs_concat_embeddings = covs_concat_embeddings_layer(
+            [subject_embeddings, covs_mode_embeddings]
+        )
+
+        # Get the covariance deviation maps (no global magnitude information)
+        covs_dev_map_input = covs_dev_map_input_layer(covs_concat_embeddings)
+        covs_dev_map = covs_dev_map_layer(covs_dev_map_input)
+        norm_covs_dev_map = norm_covs_dev_map_layer(covs_dev_map)
+
+        # Get the deviation magnitudes (scale deviation maps globally)
+        covs_dev_mag_inf_alpha_input = covs_dev_mag_inf_alpha_input_layer(data)
+        covs_dev_mag_inf_alpha = covs_dev_mag_inf_alpha_layer(
+            covs_dev_mag_inf_alpha_input
+        )
+        covs_dev_mag_inf_beta_input = covs_dev_mag_inf_beta_input_layer(data)
+        covs_dev_mag_inf_beta = covs_dev_mag_inf_beta_layer(covs_dev_mag_inf_beta_input)
+        covs_dev_mag = covs_dev_mag_layer(
+            [covs_dev_mag_inf_alpha, covs_dev_mag_inf_beta]
+        )
+        covs_dev = covs_dev_layer([covs_dev_mag, norm_covs_dev_map])
     else:
         covs_dev_layer = ZeroLayer(
             shape=(
@@ -602,62 +653,42 @@ def _model_structure(config):
             ),
             name="covs_dev",
         )
+        covs_dev = covs_dev_layer(data)
 
+    # ----------------------------------------
+    # Add deviations to group level parameters
+
+    # Layer definitions
     subject_means_layer = SubjectMapLayer(
-        "means", config.theta_std_epsilon, name="subject_means"
+        "means", config.covariances_epsilon, name="subject_means"
     )
     subject_covs_layer = SubjectMapLayer(
-        "covariances", config.theta_std_epsilon, name="subject_covs"
+        "covariances", config.covariances_epsilon, name="subject_covs"
     )
-    mix_subject_means_covs_layer = MixSubjectEmbeddingParametersLayer(
-        name="mix_subject_means_covs"
-    )
-    ll_loss_layer = LogLikelihoodLossLayer(config.theta_std_epsilon, name="ll_loss")
 
     # Data flow
-    subjects = subjects_layer(data)  # data not used here
-    subject_embeddings = subject_embedding_layer(subjects)
-
-    group_mu = group_means_layer(data)  # data not used
-    group_D = group_covs_layer(data)  # data not used
-
-    # spatial map embeddings
-    means_mode_embedding = means_mode_embedding_layer(group_mu)
-    covs_mode_embedding = covs_mode_embedding_layer(
-        InverseCholeskyLayer(config.covariances_epsilon)(group_D)
-    )
-
-    # Now get the subject specific spatial maps
-    means_concat_embedding = means_concat_embedding_layer(
-        [subject_embeddings, means_mode_embedding]
-    )
-    covs_concat_embedding = covs_concat_embedding_layer(
-        [subject_embeddings, covs_mode_embedding]
-    )
-
-    means_dev_inf_input = means_dev_inf_input_layer(means_concat_embedding)
-    means_dev_inf_mu = means_dev_inf_mu_layer(means_dev_inf_input)
-    means_dev_inf_sigma = means_dev_inf_sigma_layer(means_dev_inf_input)
-    means_dev = means_dev_layer([means_dev_inf_mu, means_dev_inf_sigma])
-
-    covs_dev_inf_input = covs_dev_inf_input_layer(covs_concat_embedding)
-    covs_dev_inf_mu = covs_dev_inf_mu_layer(covs_dev_inf_input)
-    covs_dev_inf_sigma = covs_dev_inf_sigma_layer(covs_dev_inf_input)
-    covs_dev = covs_dev_layer([covs_dev_inf_mu, covs_dev_inf_sigma])
-
     mu = subject_means_layer([group_mu, means_dev])
     D = subject_covs_layer([group_D, covs_dev])
 
-    # Mix with the mode time course
+    # -----------------------------------
+    # Mix the subject specific paraemters
+    # and get the conditional likelihood
+
+    # Layer definitions
+    mix_subject_means_covs_layer = MixSubjectSpecificParametersLayer(
+        name="mix_subject_means_covs"
+    )
+    ll_loss_layer = LogLikelihoodLossLayer(config.covariances_epsilon, name="ll_loss")
+
+    # Data flow
     m, C = mix_subject_means_covs_layer([alpha, mu, D, subj_id])
     ll_loss = ll_loss_layer([data, m, C])
 
-    # Model RNN
-    # p(theta_t | theta_<t, s_<t) ~ N(theta_t | mod_mu, mod_sigma), where
-    #   mod_mu ~ affine( Concat( RNN(theta_<t), s_t) )
-    #   mod_sigma ~ softplus(affine( Concat( RNN(theta_<t), s_t) ))
+    # ---------
+    # KL losses
 
-    # Definition of layers
+    # For the time courses (dynamic KL loss)
+    # Layer definitions
     model_input_dropout_layer = layers.Dropout(
         config.model_dropout, name="theta_norm_drop"
     )
@@ -677,73 +708,99 @@ def _model_structure(config):
         config.n_modes, activation="softplus", name="mod_sigma"
     )
     kl_div_layer = KLDivergenceLayer(config.theta_std_epsilon, name="kl_div")
-    kl_loss_layer = KLLossLayer(config.do_kl_annealing, name="kl_loss")
 
     # Data flow
     model_input_dropout = model_input_dropout_layer(theta_norm)
     model_output = model_output_layer(model_input_dropout)
-    dynamic_subject_embedding = subject_embedding_layer(subj_id)
-    model_output_concat = concatenate_layer([model_output, dynamic_subject_embedding])
+    dynamic_subject_embeddings = subject_embeddings_layer(subj_id)
+    model_output_concat = concatenate_layer([model_output, dynamic_subject_embeddings])
     mod_mu = mod_mu_layer(model_output_concat)
     mod_sigma = mod_sigma_layer(model_output_concat)
     kl_div = kl_div_layer([inf_mu, inf_sigma, mod_mu, mod_sigma])
 
-    # Prior part of deviations
-    means_dev_mod_input_layer = MultiLayerPerceptronLayer(
-        config.dev_inf_n_layers,
-        config.dev_inf_n_units,
-        config.dev_inf_normalization,
-        config.dev_inf_activation,
-        config.dev_inf_dropout,
-        name="means_dev_mod_input",
-    )
-    means_dev_mod_sigma_layer = layers.Dense(
-        config.n_channels, activation="softplus", name="means_dev_mod_sigma"
-    )
-    covs_dev_mod_input_layer = MultiLayerPerceptronLayer(
-        config.dev_inf_n_layers,
-        config.dev_inf_n_units,
-        config.dev_inf_normalization,
-        config.dev_inf_activation,
-        config.dev_inf_dropout,
-        name="covs_dev_mod_input",
-    )
-    covs_dev_mod_sigma_layer = layers.Dense(
-        config.n_channels * (config.n_channels + 1) // 2,
-        activation="softplus",
-        name="covs_dev_mod_sigma",
-    )
-
+    # For the observation model (static KL loss)
     if config.learn_means:
-        means_dev_kl_loss_layer = SubjectMapKLDivergenceLayer(
-            config.theta_std_epsilon, name="means_dev_kl_loss"
+        # Layer definitions
+        means_dev_mag_mod_beta_input_layer = MultiLayerPerceptronLayer(
+            config.dev_n_layers,
+            config.dev_n_units,
+            config.dev_normalization,
+            config.dev_activation,
+            config.dev_dropout,
+            name="means_dev_mag_mod_beta_input",
+        )
+        means_dev_mag_mod_beta_layer = layers.Dense(
+            1,
+            activation="softplus",
+            name="means_dev_mag_mod_beta",
+        )
+
+        means_dev_mag_kl_loss_layer = StaticKLDivergenceLayer(
+            config.covariances_epsilon, name="means_dev_mag_kl_loss"
+        )
+
+        # Data flow
+        means_dev_mag_mod_beta_input = means_dev_mag_mod_beta_input_layer(
+            means_concat_embeddings
+        )
+        means_dev_mag_mod_beta = means_dev_mag_mod_beta_layer(
+            means_dev_mag_mod_beta_input
+        )
+        means_dev_mag_kl_loss = means_dev_mag_kl_loss_layer(
+            [
+                data,
+                means_dev_mag_inf_alpha,
+                means_dev_mag_inf_beta,
+                means_dev_mag_mod_beta,
+            ]
         )
     else:
-        means_dev_kl_loss_layer = ZeroLayer((), name="means_dev_kl_loss")
+        means_dev_mag_kl_loss_layer = ZeroLayer((), name="means_dev_mag_kl_loss")
+        means_dev_mag_kl_loss = means_dev_mag_kl_loss_layer(data)
 
     if config.learn_covariances:
-        covs_dev_kl_loss_layer = SubjectMapKLDivergenceLayer(
-            config.theta_std_epsilon, name="covs_dev_kl_loss"
+        # Layer definitions
+        covs_dev_mag_mod_beta_input_layer = MultiLayerPerceptronLayer(
+            config.dev_n_layers,
+            config.dev_n_units,
+            config.dev_normalization,
+            config.dev_activation,
+            config.dev_dropout,
+            name="covs_dev_mag_mod_beta_input",
+        )
+        covs_dev_mag_mod_beta_layer = layers.Dense(
+            1,
+            activation="softplus",
+            name="covs_dev_mag_mod_beta",
+        )
+
+        covs_dev_mag_kl_loss_layer = StaticKLDivergenceLayer(
+            config.covariances_epsilon, name="covs_dev_mag_kl_loss"
+        )
+
+        # Data flow
+        covs_dev_mag_mod_beta_input = covs_dev_mag_mod_beta_input_layer(
+            covs_concat_embeddings
+        )
+        covs_dev_mag_mod_beta = covs_dev_mag_mod_beta_layer(covs_dev_mag_mod_beta_input)
+        covs_dev_mag_kl_loss = covs_dev_mag_kl_loss_layer(
+            [
+                data,
+                covs_dev_mag_inf_alpha,
+                covs_dev_mag_inf_beta,
+                covs_dev_mag_mod_beta,
+            ]
         )
     else:
-        covs_dev_kl_loss_layer = ZeroLayer((), name="covs_dev_kl_loss")
+        covs_dev_mag_kl_loss_layer = ZeroLayer((), name="covs_dev_mag_kl_loss")
+        covs_dev_mag_kl_loss = covs_dev_mag_kl_loss_layer(data)
 
-    kl_loss_layer = KLLossLayer(do_annealing=config.do_kl_annealing, name="kl_loss")
+    # Total KL loss
+    # Layer definitions
+    kl_loss_layer = KLLossLayer(config.do_kl_annealing, name="kl_loss")
 
     # Data flow
-    means_dev_mod_input = means_dev_mod_input_layer(means_concat_embedding)
-    means_dev_mod_sigma = means_dev_mod_sigma_layer(means_dev_mod_input)
-
-    covs_dev_mod_input = covs_dev_mod_input_layer(covs_concat_embedding)
-    covs_dev_mod_sigma = covs_dev_mod_sigma_layer(covs_dev_mod_input)
-
-    means_dev_kl_loss = means_dev_kl_loss_layer(
-        [data, means_dev_inf_mu, means_dev_inf_sigma, means_dev_mod_sigma]
-    )
-    covs_dev_kl_loss = covs_dev_kl_loss_layer(
-        [data, covs_dev_inf_mu, covs_dev_inf_sigma, covs_dev_mod_sigma]
-    )
-    kl_loss = kl_loss_layer([kl_div, means_dev_kl_loss, covs_dev_kl_loss])
+    kl_loss = kl_loss_layer([kl_div, means_dev_mag_kl_loss, covs_dev_mag_kl_loss])
 
     return tf.keras.Model(
         inputs=[data, subj_id],
